#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <GL/glut.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <time.h>
#include "mathlink.h"

#define DIM 25
#define DS DIM*DIM
#define FR 4

extern int connectToServer(int port, char* IP);
extern double * getNewFrame(void);
extern int addForces(int spx1, int spy1, double d_fx1, double d_fy1);
extern int sendRequestToTerminate(void);
extern int sendEnvironmentVariables(void);

float particles[DS*2];
double d_particles[2*DS];
int lastx,lasty;
int clicked=0;
int wWidth=512, wHeight=512;
int sockfd;
int totalBytes=0; 
int frames=0;
time_t start,end;

/**********Network Functions************/
int connectToServer(int port, char* IP) {
   fprintf(stderr,"Attempting to connect to server...\n");
   struct sockaddr_in remote_addr;

   sockfd=socket(PF_INET,SOCK_STREAM,0);
   
   remote_addr.sin_family=AF_INET;
   remote_addr.sin_port=htons(port);
   remote_addr.sin_addr.s_addr=inet_addr(IP);
   memset(remote_addr.sin_zero, '\0',sizeof remote_addr.sin_zero);

   if(connect(sockfd,(struct sockaddr*)&remote_addr,sizeof remote_addr)==-1){
      perror("connect");
      exit(1);
   }
   
   fprintf(stderr,"Connected to server.\n");
   return 1;
}

double * getNewFrame(void) {
   fprintf(stderr,"Requesting new frame...\n");

   char newFrameRequest[17];
   newFrameRequest[0]=0x1;
   memset(&newFrameRequest[1], '\0',16);
   send(sockfd,newFrameRequest,17,0);

   int bytesRecieved=recv(sockfd,particles,DS*2*sizeof(float),MSG_WAITALL);
   if(bytesRecieved==-1) { perror("recv()"); exit(1); }

   fprintf(stderr,"Recieved frame. (%d bytes).\n",bytesRecieved);
   totalBytes+=bytesRecieved;
   ++frames;

   int i;
   for(i = 0; i < 2*DS; i++) {
       d_particles[i] = particles[i];
   }
   MLPutRealList(stdlink, (double *) d_particles, 2*DS);
   return ;
}

int addForces(int spx1, int spy1, double d_fx1, double d_fy1) {
   float fx1 = d_fx1;
   float fy1 = d_fy1;
   fprintf(stderr,"Sending force data...\n");
   char addForceBuffer[17];
   addForceBuffer[0]=0x2;
   int spx=spx1,spy=spy1;
   memcpy(&addForceBuffer[1],&spx,4);
   memcpy(&addForceBuffer[5],&spy,4);
   float fx=fx1,fy=fy1;
   memcpy(&addForceBuffer[9],&fx,4);
   memcpy(&addForceBuffer[13],&fy,4);

   send(sockfd,addForceBuffer,17,0);
   fprintf(stderr,"Force data sent.\n");
   return 1;
}

int sendRequestToTerminate(void) {
   char terminate[17];
   terminate[0]=0x3;
   memset(&terminate[1],'\0',16);
   send(sockfd,terminate,17,0);
   return 1;
}
/***************************************/


int sendEnvironmentVariables(void) {
   fprintf(stderr,"Sending environment data...\n");
   int dim=DIM;
   send(sockfd,&dim,sizeof(int),0);
   fprintf(stderr,"Sent environment data.\n");
   return 1;
}


#if MACINTOSH_MATHLINK

int main( int argc, char* argv[])
{
	/* Due to a bug in some standard C libraries that have shipped with
	 * MPW, zero is passed to MLMain below.  (If you build this program
	 * as an MPW tool, you can change the zero to argc.)
	 */
	argc = argc; /* suppress warning */
	return MLMain( 0, argv);
}

#elif WINDOWS_MATHLINK

#if __BORLANDC__
#pragma argsused
#endif

int PASCAL WinMain( HINSTANCE hinstCurrent, HINSTANCE hinstPrevious, LPSTR lpszCmdLine, int nCmdShow)
{
	char  buff[512];
	char FAR * buff_start = buff;
	char FAR * argv[32];
	char FAR * FAR * argv_end = argv + 32;

	hinstPrevious = hinstPrevious; /* suppress warning */

	if( !MLInitializeIcon( hinstCurrent, nCmdShow)) return 1;
	MLScanString( argv, &argv_end, &lpszCmdLine, &buff_start);
	return MLMain( argv_end - argv, argv);
}

#else

int main(int argc, char* argv[])
{
	return MLMain(argc, argv);
}

#endif
